@@ -43,7 +43,8 @@ class Grilo(GObject.GObject):
         Grl.METADATA_KEY_ID, Grl.METADATA_KEY_TITLE,
         Grl.METADATA_KEY_ARTIST, Grl.METADATA_KEY_ALBUM,
         Grl.METADATA_KEY_DURATION,
-        Grl.METADATA_KEY_CREATION_DATE]
+        Grl.METADATA_KEY_CREATION_DATE,
+        Grl.METADATA_KEY_THUMBNAIL]
 
     METADATA_THUMBNAIL_KEYS = [
         Grl.METADATA_KEY_ID,
@@ -61,9 +62,13 @@ class Grilo(GObject.GObject):
         if not (GLib.file_test(self.playlist_path, GLib.FileTest.IS_DIR)):
             GLib.mkdir_with_parents(self.playlist_path, int("0755", 8))
         self.options = Grl.OperationOptions()
-        self.options.set_flags(Grl.ResolutionFlags.FULL |
+        self.options.set_flags(Grl.ResolutionFlags.FAST_ONLY |
                                Grl.ResolutionFlags.IDLE_RELAY)
 
+        self.full_options = Grl.OperationOptions()
+        self.full_options.set_flags(Grl.ResolutionFlags.FULL |
+                                    Grl.ResolutionFlags.IDLE_RELAY)
+
         self.sources = {}
         self.tracker = None
         self.changed_media_ids = []
@@ -142,29 +147,29 @@ class Grilo(GObject.GObject):
 
     @log
     def populate_artists(self, offset, callback, count=-1):
-        self.populate_items(Query.get_artists(), offset, callback, count)
+        self.populate_items(Query.ARTISTS, offset, callback, count)
 
     @log
-    def populate_albums(self, offset, callback, count=50):
-        self.populate_items(Query.get_all_albums(), offset, callback, count)
+    def populate_albums(self, offset, callback, count=-1):
+        self.populate_items(Query.ALBUMS, offset, callback, count)
 
     @log
     def populate_songs(self, offset, callback, count=-1):
-        self.populate_items(Query.get_songs(), offset, callback, count)
+        self.populate_items(Query.SONGS, offset, callback, count)
 
     @log
     def populate_album_songs(self, album_id, callback, count=-1):
         self.populate_items(Query.album_songs(album_id), 0, callback, count)
 
     @log
-    def populate_items(self, query, offset, callback, count=50):
+    def populate_items(self, query, offset, callback, count=-1):
         options = self.options.copy()
         options.set_skip(offset)
         if count != -1:
             options.set_count(count)
 
-        def _callback(source, param, item, count, data, offset):
-            callback(source, param, item, count)
+        def _callback(source, param, item, remaining, data, offset):
+            callback(source, param, item, remaining)
         self.tracker.query(query, self.METADATA_KEYS, options, _callback, None)
 
     @log
@@ -179,12 +184,6 @@ class Grilo(GObject.GObject):
             source.search(q, [Grl.METADATA_KEY_ID], 0, 10,
                           options, self._search_callback, source)
 
-    @log
-    def get_album_art_for_album_id(self, album_id, _callback):
-        options = self.options.copy()
-        query = Query.get_album_for_id(album_id)
-        self.tracker.query(query, self.METADATA_THUMBNAIL_KEYS, options, _callback, None)
-
     @log
     def get_media_from_uri(self, uri, callback):
         options = self.options.copy()
@@ -197,6 +196,14 @@ class Grilo(GObject.GObject):
 
         self.tracker.query(query, self.METADATA_KEYS, options, _callback, None)
 
+    @log
+    def get_album_art_for_album_id(self, album_id, callback, data=None):
+        options = self.full_options.copy()
+        options.set_count(1)
+        query = Query.get_album_for_id(album_id)
+        self.tracker.query(query, self.METADATA_THUMBNAIL_KEYS, options, callback, data)
+
+
 Grl.init(None)
 
 grilo = Grilo()
