@@ -85,7 +85,7 @@ class Player(GObject.GObject):
         self.currentTrack = None
         self._lastState = Gst.State.PAUSED
         self.cache = AlbumArtCache.get_default()
-        self._symbolicIcon = self.cache.make_default_icon(ART_SIZE, ART_SIZE)
+        self._symbolicIcon = self.cache.get_default_icon(ART_SIZE, ART_SIZE)
 
         Gst.init(None)
 
@@ -376,18 +376,26 @@ class Player(GObject.GObject):
         self.playBtn.set_sensitive(True)
         self._sync_prev_next()
 
+        artist = _("Unknown Artist")
+        try:
+            assert media.get_artist() is not None
+            artist = media.get_artist()
+        finally:
+            self.artistLabel.set_label(artist)
+
+        album = _("Unknown Album")
+        try:
+            assert media.get_album() is not None
+            album = media.get_album()
+        except:
+            pass
+
         self.coverImg.set_from_pixbuf(self._symbolicIcon)
-        self.cache.lookup(media, ART_SIZE, ART_SIZE, self._on_cache_lookup)
+        self.cache.lookup(
+            media, ART_SIZE, ART_SIZE, self._on_cache_lookup, None, artist, album)
 
         self.titleLabel.set_label(AlbumArtCache.get_media_title(media))
 
-        try:
-            artist = media.get_artist()
-            assert artist is not None
-            self.artistLabel.set_label(artist)
-        except:
-            self.artistLabel.set_label(_("Unknown Artist"))
-
         url = media.get_url()
         if url != self.player.get_value('current-uri', 0):
             self.player.set_property('uri', url)
@@ -466,12 +474,14 @@ class Player(GObject.GObject):
             return
 
         position = self.get_position() / 1000000
+        if position >= 5:
+            self.progressScale.set_value(0)
+            self.on_progress_scale_change_value(self.progressScale)
+            return
 
         self.stop()
 
-        if position < 5:
-            self.currentTrack = self._get_previous_track()
-
+        self.currentTrack = self._get_previous_track()
         if self.currentTrack:
             self.play()
 
