@@ -32,6 +32,7 @@
 
 
 from gi.repository import Gtk
+from gi.repository import Gdk
 from gi.repository import GObject
 from gi.repository import Gd
 from gi.repository import Grl
@@ -39,14 +40,12 @@ from gi.repository import Pango
 from gi.repository import GLib
 from gi.repository import GdkPixbuf
 from gi.repository import Tracker
-from gi.repository import Gio
 
 from gettext import gettext as _, ngettext
 from gnomemusic.grilo import grilo
 from gnomemusic.toolbar import ToolbarState
 import gnomemusic.widgets as Widgets
 from gnomemusic.playlists import Playlists
-from gnomemusic.query import Query
 from gnomemusic.albumArtCache import AlbumArtCache as albumArtCache
 from gnomemusic import log
 import logging
@@ -61,26 +60,20 @@ except Exception as e:
 
 playlists = Playlists.get_default()
 
-if Gtk.get_minor_version() > 8:
-    from gi.repository.Gtk import Stack, StackTransitionType
-else:
-    from gi.repository.Gd import Stack, StackTransitionType
 
-
-class ViewContainer(Stack):
+class ViewContainer(Gtk.Stack):
     if Gtk.Widget.get_default_direction() is not Gtk.TextDirection.RTL:
         nowPlayingIconName = 'media-playback-start-symbolic'
     else:
         nowPlayingIconName = 'media-playback-start-rtl-symbolic'
     errorIconName = 'dialog-error-symbolic'
     starIconName = 'starred-symbolic'
-    countQuery = None
     filter = None
 
     @log
     def __init__(self, title, header_bar, selection_toolbar, view_type, use_sidebar=False, sidebar=None):
-        Stack.__init__(self,
-                       transition_type=StackTransitionType.CROSSFADE)
+        Gtk.Stack.__init__(self,
+                           transition_type=Gtk.StackTransitionType.CROSSFADE)
         self._grid = Gtk.Grid(orientation=Gtk.Orientation.HORIZONTAL)
         self._iconWidth = 128
         self._iconHeight = 128
@@ -88,6 +81,7 @@ class ViewContainer(Stack):
         self._adjustmentValueId = 0
         self._adjustmentChangedId = 0
         self._scrollbarVisibleId = 0
+        self.old_vsbl_range = None
         self._model = Gtk.ListStore(
             GObject.TYPE_STRING,
             GObject.TYPE_STRING,
@@ -112,8 +106,8 @@ class ViewContainer(Stack):
         box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         box.pack_start(self.view, True, True, 0)
         if use_sidebar:
-            self.stack = Stack(
-                transition_type=StackTransitionType.SLIDE_RIGHT,
+            self.stack = Gtk.Stack(
+                transition_type=Gtk.StackTransitionType.SLIDE_RIGHT,
             )
             dummy = Gtk.Frame(visible=False)
             self.stack.add_named(dummy, 'dummy')
@@ -126,10 +120,6 @@ class ViewContainer(Stack):
         if not use_sidebar or sidebar:
             self._grid.add(box)
 
-        self._cached_count = -1
-        self._loadMore = Widgets.LoadMoreButton(self._get_remaining_item_count)
-        box.pack_end(self._loadMore.widget, False, False, 0)
-        self._loadMore.widget.connect('clicked', self._populate)
         self.view.connect('item-activated', self._on_item_activated)
         self.view.connect('selection-mode-request', self._on_selection_mode_request)
         self._cursor = None
@@ -144,11 +134,10 @@ class ViewContainer(Stack):
 
         self.show_all()
         self._items = []
-        self._loadMore.widget.hide()
         self._connect_view()
         self.cache = albumArtCache.get_default()
-        self._symbolicIcon = self.cache.make_default_icon(self._iconHeight,
-                                                          self._iconWidth)
+        self._symbolicIcon = self.cache.get_default_icon(self._iconHeight,
+                                                         self._iconWidth)
 
         self._init = False
         grilo.connect('ready', self._on_grilo_ready)
@@ -166,23 +155,19 @@ class ViewContainer(Stack):
     def _on_changes_pending(self, data=None):
         pass
 
-    @log
-    def _get_remaining_item_count(self):
-        if self._cached_count < 0:
-            self._cached_count = Widgets.get_count(self.countQuery)
-        return self._cached_count - self._offset
-
     @log
     def _on_header_bar_toggled(self, button):
         if button.get_active():
             self.view.set_selection_mode(True)
             self.header_bar.set_selection_mode(True)
+            self.player.eventBox.set_visible(False)
             self.selection_toolbar.eventbox.set_visible(True)
             self.selection_toolbar._add_to_playlist_button.set_sensitive(False)
             self.selection_toolbar._remove_from_playlist_button.set_sensitive(False)
         else:
             self.view.set_selection_mode(False)
             self.header_bar.set_selection_mode(False)
+            self.player.eventBox.set_visible(self.player.currentTrack is not None)
             self.selection_toolbar.eventbox.set_visible(False)
 
     @log
@@ -231,33 +216,53 @@ class ViewContainer(Stack):
 
     @log
     def _connect_view(self):
-        self._adjustmentValueId = self.vadjustment.connect(
-            'value-changed',
-            self._on_scrolled_win_change)
+        self.view.connect_after('draw', self._on_view_draw)
+        self.view.add_events(Gdk.EventMask.EXPOSURE_MASK)
 
     @log
-    def _on_scrolled_win_change(self, data=None):
-        vScrollbar = self.view.get_vscrollbar()
-        revealAreaHeight = 32
+    def _on_view_draw(self, widget, cr):
+        vsbl_range = widget.get_children()[0].get_visible_range()
+        if not vsbl_range or self.old_vsbl_range == vsbl_range:
+            return
+        self.old_vsbl_range = vsbl_range
+        self.on_scroll_event(self.view)
 
-        # if there's no vscrollbar, or if it's not visible, hide the button
-        if not vScrollbar or not vScrollbar.get_visible():
-            self._loadMore.set_block(True)
+    @log
+    def on_scroll_event(self, widget, event=None):
+        vsbl_range = widget.get_children()[0].get_visible_range()
+        if not vsbl_range:
             return
 
-        value = self.vadjustment.get_value()
-        upper = self.vadjustment.get_upper()
-        page_size = self.vadjustment.get_page_size()
+        def load_album_art_for(path):
+            try:
+                _iter = self._model.get_iter(path)
+                item = self._model.get_value(_iter, 5)
+                if not item:
+                    return
+                title = self._model.get_value(_iter, 2)
+                artist = self._model.get_value(_iter, 3)
+                thumbnail = self._model.get_value(_iter, 4)
+                if thumbnail == self._symbolicIcon:
+                    albumArtCache.get_default().lookup(
+                        item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
+                        _iter, artist, title)
+            except Exception:
+                pass
 
-        end = False
-        # special case self values which happen at construction
-        if (value == 0) and (upper == 1) and (page_size == 1):
-            end = False
-        else:
-            end = not (value < (upper - page_size - revealAreaHeight))
-        if self._get_remaining_item_count() <= 0:
-            end = False
-        self._loadMore.set_block(not end)
+        # Load thumbnails
+        path = vsbl_range[0]
+        while path <= vsbl_range[1]:
+            load_album_art_for(path)
+            path.next()
+
+        # Add 10 more albums to avoid visible thumbnail loading
+        for i in range(0, 10):
+            try:
+                path.next()
+                load_album_art_for(path)
+            except ValueError:
+                # No such path
+                break
 
     @log
     def populate(self):
@@ -294,35 +299,12 @@ class ViewContainer(Stack):
                             [str(item.get_id()), '', title,
                              artist, self._symbolicIcon, item,
                              0, icon_name, False, icon_name == self.errorIconName])
-            GLib.idle_add(self._update_album_art, item, _iter)
-
         GLib.idle_add(add_new_item)
 
-    @log
-    def _insert_album_art(self, item, cb_item, itr, x=False):
-        if item and cb_item and not item.get_thumbnail():
-            if cb_item.get_thumbnail():
-                item.set_thumbnail(cb_item.get_thumbnail())
-            albumArtCache.get_default().lookup(
-                item,
-                self._iconWidth,
-                self._iconHeight,
-                self._on_lookup_ready, itr)
-
-    @log
-    def _update_album_art(self, item, itr):
-        grilo.get_album_art_for_album_id(
-            item.get_id(),
-            lambda source, count, cb_item, x, y, z:
-            self._insert_album_art(item, cb_item, itr, True)
-        )
-
     @log
     def _on_lookup_ready(self, icon, path, _iter):
         if icon:
-            self._model.set_value(
-                _iter, 4,
-                albumArtCache.get_default()._make_icon_frame(icon))
+            self._model.set_value(_iter, 4, icon)
             self.view.queue_draw()
 
     @log
@@ -343,11 +325,11 @@ class ViewContainer(Stack):
 
 
 # Class for the Empty View
-class Empty(Stack):
+class Empty(Gtk.Stack):
     @log
     def __init__(self, header_bar, player):
-        Stack.__init__(self,
-                       transition_type=StackTransitionType.CROSSFADE)
+        Gtk.Stack.__init__(self,
+                           transition_type=Gtk.StackTransitionType.CROSSFADE)
         builder = Gtk.Builder()
         builder.add_from_resource('/org/gnome/Music/NoMusic.ui')
         music_folder_path = GLib.get_user_special_dir(GLib.USER_DIRECTORY_MUSIC)
@@ -363,7 +345,6 @@ class Albums(ViewContainer):
     def __init__(self, header_bar, selection_toolbar, player):
         ViewContainer.__init__(self, _("Albums"), header_bar,
                                selection_toolbar, Gd.MainViewType.ICON)
-        self.countQuery = Query.get_albums_count()
         self._albumWidget = Widgets.AlbumWidget(player)
         self.player = player
         self.add(self._albumWidget)
@@ -375,7 +356,6 @@ class Albums(ViewContainer):
     def _on_changes_pending(self, data=None):
         if (self._init):
             self._offset = 0
-            self._cached_count = -1
             self._model.clear()
             self.populate()
 
@@ -442,9 +422,7 @@ class Songs(ViewContainer):
     @log
     def __init__(self, header_bar, selection_toolbar, player):
         ViewContainer.__init__(self, _("Songs"), header_bar, selection_toolbar, Gd.MainViewType.LIST)
-        self.countQuery = Query.get_songs_count()
         self._items = {}
-        self.monitors = []
         self.isStarred = None
         self.iter_to_clean = None
         self.view.get_generic_view().get_style_context()\
@@ -452,8 +430,8 @@ class Songs(ViewContainer):
         self._iconHeight = 32
         self._iconWidth = 32
         self.cache = albumArtCache.get_default()
-        self._symbolicIcon = self.cache.make_default_icon(self._iconHeight,
-                                                          self._iconWidth)
+        self._symbolicIcon = self.cache.get_default_icon(self._iconHeight,
+                                                         self._iconWidth)
         self._add_list_renderers()
         self.player = player
         self.player.connect('playlist-item-changed', self.update_model)
@@ -463,7 +441,6 @@ class Songs(ViewContainer):
         if (self._init):
             self._model.clear()
             self._offset = 0
-            self._cached_count = -1
             self.populate()
 
     @log
@@ -504,16 +481,6 @@ class Songs(ViewContainer):
             [albumArtCache.get_media_title(item),
              artist, item, self.nowPlayingIconName, False, False])
         self.player.discover_item(item, self._on_discovered, _iter)
-        g_file = Gio.file_new_for_uri(item.get_url())
-        self.monitors.append(g_file.monitor_file(Gio.FileMonitorFlags.NONE,
-                                                 None))
-        self.monitors[(self._offset - 1)].connect('changed',
-                                                  self._on_item_changed, _iter)
-
-    def _on_item_changed(self, monitor, file1, file2, event, _iter):
-        if self._model.iter_is_valid(_iter):
-            if event == Gio.FileMonitorEvent.DELETED:
-                self._model.set(_iter, [8, 10], [self.errorIconName, True])
 
     @log
     def _add_list_renderers(self):
@@ -620,9 +587,8 @@ class Artists (ViewContainer):
         self.albums_selected = []
         self.items_selected = []
         self.items_selected_callback = None
-        self.countQuery = Query.ARTISTS_COUNT
-        self.artistAlbumsStack = Stack(
-            transition_type=StackTransitionType.CROSSFADE,
+        self.artistAlbumsStack = Gtk.Stack(
+            transition_type=Gtk.StackTransitionType.CROSSFADE,
         )
         self._artistAlbumsWidget = Gtk.Frame(
             shadow_type=Gtk.ShadowType.NONE,
@@ -651,7 +617,6 @@ class Artists (ViewContainer):
             self._model.clear()
             self._artists.clear()
             self._offset = 0
-            self._cached_count = -1
             self._populate()
 
     @log
@@ -676,16 +641,16 @@ class Artists (ViewContainer):
         cells = cols[0].get_cells()
         cells[1].set_visible(False)
         cells[2].set_visible(False)
-        type_renderer = Gd.StyledTextRenderer(
+        self.text_renderer = Gd.StyledTextRenderer(
             xpad=16,
             ypad=16,
             ellipsize=Pango.EllipsizeMode.END,
             xalign=0.0,
             width=220
         )
-        list_widget.add_renderer(type_renderer, lambda *args: None, None)
-        cols[0].clear_attributes(type_renderer)
-        cols[0].add_attribute(type_renderer, 'text', 2)
+        list_widget.add_renderer(self.text_renderer, lambda *args: None, None)
+        cols[0].clear_attributes(self.text_renderer)
+        cols[0].add_attribute(self.text_renderer, 'text', 2)
 
     @log
     def _on_item_activated(self, widget, item_id, path):
@@ -752,11 +717,13 @@ class Artists (ViewContainer):
         ViewContainer._on_header_bar_toggled(self, button)
 
         if button.get_active():
+            self.text_renderer.set_fixed_size(178, -1)
             self._last_selection =\
                 self.view.get_generic_view().get_selection().get_selected()[1]
             self.view.get_generic_view().get_selection().set_mode(
                 Gtk.SelectionMode.NONE)
         else:
+            self.text_renderer.set_fixed_size(220, -1)
             self.view.get_generic_view().get_selection().set_mode(
                 Gtk.SelectionMode.SINGLE)
             if self._last_selection is not None:
@@ -860,7 +827,6 @@ class Playlist(ViewContainer):
             self.playlists_sidebar.get_generic_view().get_style_context().\
                 add_class("artist-panel-white")
 
-        self.monitors = []
         self.iter_to_clean = None
         self.iter_to_clean_model = None
         self.current_playlist = None
@@ -1007,16 +973,14 @@ class Playlist(ViewContainer):
             self.player.set_playlist('Playlist', self.current_playlist, self.filter, _iter, 5)
             self.player.set_playing(True)
 
-    @log
-    def _on_item_changed(self, monitor, file1, file2, event, _iter):
-        if self._model.iter_is_valid(_iter):
-            if event == Gio.FileMonitorEvent.DELETED:
-                self._model.set(_iter, [8, 10], [self.errorIconName, True])
-
     @log
     def _on_playlist_activated(self, widget, item_id, path):
         _iter = self.playlists_model.get_iter(path)
         playlist = self.playlists_model.get_value(_iter, 2)
+
+        if self.current_playlist == playlist:
+            return
+
         self.current_playlist = playlist
         self.name_label.set_text(playlist)
 
@@ -1071,11 +1035,6 @@ class Playlist(ViewContainer):
             [albumArtCache.get_media_title(item),
              artist, item, self.nowPlayingIconName, False, False])
         self.player.discover_item(item, self._on_discovered, _iter)
-        g_file = Gio.file_new_for_uri(item.get_url())
-        self.monitors.append(g_file.monitor_file(Gio.FileMonitorFlags.NONE,
-                                                 None))
-        self.monitors[(self._offset - 1)].connect('changed',
-                                                  self._on_item_changed, _iter)
         self.songs_count += 1
         self._update_songs_count()
 
@@ -1108,6 +1067,14 @@ class Playlist(ViewContainer):
         if not _iter:
             return
 
+        iter_next = self.playlists_model.iter_next(_iter)\
+            or self.playlists_model.iter_previous(_iter)
+        if iter_next:
+            selection = self.playlists_sidebar.get_generic_view().get_selection()
+            selection.select_iter(iter_next)
+            self.playlists_sidebar.emit('item-activated', '0',
+                                        self.playlists_model.get_path(iter_next))
+
         playlist = self.playlists_model.get_value(_iter, 2)
         playlists.delete_playlist(playlist)
         self.playlists_model.remove(_iter)
@@ -1115,6 +1082,12 @@ class Playlist(ViewContainer):
     @log
     def _on_playlist_created(self, playlists, name):
         self._add_playlist_item(name)
+        if self.playlists_model.iter_n_children(None) == 1:
+            _iter = self.playlists_model.get_iter_first()
+            selection = self.playlists_sidebar.get_generic_view().get_selection()
+            selection.select_iter(_iter)
+            self.playlists_sidebar.emit('item-activated', '0',
+                                        self.playlists_model.get_path(_iter))
 
     @log
     def _on_song_added_to_playlist(self, playlists, name, item):
@@ -1174,6 +1147,12 @@ class Playlist(ViewContainer):
     def populate(self):
         for item in sorted(self.playlists_list):
             self._add_playlist_item(item)
+        if len(self.playlists_list):
+            _iter = self.playlists_model.get_iter_first()
+            selection = self.playlists_sidebar.get_generic_view().get_selection()
+            selection.select_iter(_iter)
+            self.playlists_sidebar.emit('item-activated', '0',
+                                        self.playlists_model.get_path(_iter))
 
     @log
     def get_selected_track_uris(self, callback):
