@@ -32,11 +32,10 @@
 
 
 from gi.repository import Gtk, Gd, GLib, GObject, Pango
-from gi.repository import GdkPixbuf, Gio
+from gi.repository import GdkPixbuf, Gio, Grl
 from gi.repository import Tracker
 from gettext import gettext as _, ngettext
 from gnomemusic.grilo import grilo
-from gnomemusic.query import Query
 from gnomemusic.albumArtCache import AlbumArtCache
 from gnomemusic.playlists import Playlists
 from gnomemusic import log
@@ -60,69 +59,11 @@ else:
 ERROR_ICON_NAME = 'dialog-error-symbolic'
 
 
-@log
-def get_count(countQuery):
-    count = -1
-    if countQuery:
-        cursor = tracker.query(countQuery, None)
-        if cursor and cursor.next(None):
-            count = cursor.get_integer(0)
-    return count
-
-
-class LoadMoreButton:
-    @log
-    def __init__(self, counter):
-        self._block = False
-        self._counter = counter
-        child = Gtk.Grid(column_spacing=10,
-                         hexpand=False,
-                         halign=Gtk.Align.CENTER,
-                         visible=True)
-        self._spinner = Gtk.Spinner(halign=Gtk.Align.CENTER,
-                                    no_show_all=True)
-        self._spinner.set_size_request(16, 16)
-        child.add(self._spinner)
-        self._label = Gtk.Label(label=_("Load More"),
-                                visible=True)
-        child.add(self._label)
-        self.widget = Gtk.Button(no_show_all=True,
-                                 child=child)
-        self.widget.get_style_context().add_class('documents-load-more')
-        self.widget.connect('clicked', self._on_load_more_clicked)
-        self._on_item_count_changed()
-
-    @log
-    def _on_load_more_clicked(self, data=None):
-        self._label.set_label(_("Loading..."))
-        self._spinner.show()
-        self._spinner.start()
-
-    @log
-    def _on_item_count_changed(self):
-        remaining_docs = self._counter()
-        visible = remaining_docs >= 0 and not self._block
-        self.widget.set_visible(visible)
-
-        if visible:
-            self._label.set_label(_("Load More"))
-            self._spinner.stop()
-            self._spinner.hide()
-
-    @log
-    def set_block(self, block):
-        if (self._block == block):
-            return
-
-        self._block = block
-        self._on_item_count_changed()
-
-
 class AlbumWidget(Gtk.EventBox):
 
     tracks = []
     duration = 0
-    symbolicIcon = ALBUM_ART_CACHE.make_default_icon(256, 256)
+    symbolicIcon = ALBUM_ART_CACHE.get_default_icon(256, 256)
     filter = None
 
     @log
@@ -235,9 +176,11 @@ class AlbumWidget(Gtk.EventBox):
         self.selection_toolbar = selection_toolbar
         self.header_bar = header_bar
         self.album = album
+        real_artist = item.get_string(Grl.METADATA_KEY_ARTIST)\
+            or item.get_author()\
+            or _("Unknown Artist")
         self.ui.get_object('cover').set_from_pixbuf(self.symbolicIcon)
-        ALBUM_ART_CACHE.lookup(item, 256, 256,
-                               self._on_look_up)
+        ALBUM_ART_CACHE.lookup(item, 256, 256, self._on_look_up, None, real_artist, album)
 
         # if the active queue has been set by self album,
         # use it as model, otherwise build the liststore
@@ -483,59 +426,13 @@ class AllArtistsAlbums(ArtistAlbums):
     def __init__(self, player):
         ArtistAlbums.__init__(self, _("All Artists"), [], player)
         self._offset = 0
-        self.countQuery = Query.get_albums_count()
-        self._cached_count = -1
-        self._load_more = LoadMoreButton(self._get_remaining_item_count)
-        self.pack_end(self._load_more.widget, False, False, 0)
-        self._load_more.widget.connect('clicked', self._populate)
-        self.vadjustment = self._scrolledWindow.get_vadjustment()
-        self._connect_view()
         self._populate()
 
-    @log
-    def _get_remaining_item_count(self):
-        if self._cached_count < 0:
-            self._cached_count = get_count(self.countQuery)
-        return self._cached_count - self._offset
-
-    @log
-    def _connect_view(self):
-        self._adjustmentValueId =\
-            self.vadjustment.connect('value-changed', self._on_scrolled_win_change)
-        self._adjustmentChangedId =\
-            self.vadjustment.connect('changed', self._on_scrolled_win_change)
-        self._scrollbarVisibleId =\
-            self._scrolledWindow.get_vscrollbar().connect(
-                'notify::visible',
-                self._on_scrolled_win_change)
-        self._on_scrolled_win_change()
-
-    @log
-    def _on_scrolled_win_change(self, scrollbar=None, pspec=None, data=None):
-        vScrollbar = self._scrolledWindow.get_vscrollbar()
-        revealAreaHeight = 32
-
-        # if there's no vscrollbar, or if it's not visible, hide the button
-        if not vScrollbar or not vScrollbar.get_visible():
-            self._load_more.set_block(True)
-            return
-
-        value = self.vadjustment.get_value()
-        upper = self.vadjustment.get_upper()
-        page_size = self.vadjustment.get_page_size()
-        end = False
-
-        # special case this values which happen at construction
-        if (((value != 0) or (upper != 1) or (page_size != 1))
-                and self._get_remaining_item_count() > 0):
-            end = not (value < (upper - page_size - revealAreaHeight))
-        self._load_more.set_block(not end)
-
     @log
     def _populate(self, data=None):
         if grilo.tracker:
             GLib.idle_add(grilo.populate_albums,
-                          self._offset, self.add_item, 5)
+                          self._offset, self.add_item, -1)
 
     @log
     def add_item(self, source, param, item, remaining):
@@ -559,7 +456,7 @@ class ArtistAlbumWidget(Gtk.HBox):
         self.ui.add_from_resource('/org/gnome/Music/ArtistAlbumWidget.ui')
 
         self.cache = AlbumArtCache.get_default()
-        pixbuf = self.cache.make_default_icon(128, 128)
+        pixbuf = self.cache.get_default_icon(128, 128)
         GLib.idle_add(self._update_album_art)
 
         self.ui.get_object('cover').set_from_pixbuf(pixbuf)
@@ -639,7 +536,12 @@ class ArtistAlbumWidget(Gtk.HBox):
 
     @log
     def _update_album_art(self):
-        ALBUM_ART_CACHE.lookup(self.album, 128, 128, self._get_album_cover)
+        real_artist = self.album.get_string(Grl.METADATA_KEY_ARTIST)\
+            or self.album.get_author()\
+            or _("Unknown Artist")
+        ALBUM_ART_CACHE.lookup(
+            self.album, 128, 128, self._get_album_cover, None,
+            real_artist, self.album.get_title())
 
     @log
     def _get_album_cover(self, pixbuf, path, data=None):
@@ -675,11 +577,7 @@ class PlaylistDialog():
         self.populate(playlist_names)
 
         self.title_bar = self.ui.get_object('headerbar1')
-        if Gtk.get_minor_version() > 8:
-            self.dialog_box.set_titlebar(self.title_bar)
-        else:
-            self.dialog_box.get_content_area().add(self.title_bar)
-            self.dialog_box.get_content_area().reorder_child(self.title_bar, 0)
+        self.dialog_box.set_titlebar(self.title_bar)
 
         self._cancel_button = self.ui.get_object('cancel-button')
         self._select_button = self.ui.get_object('select-button')
@@ -700,16 +598,16 @@ class PlaylistDialog():
     def _add_list_renderers(self):
         cols = Gtk.TreeViewColumn()
         type_renderer = Gd.StyledTextRenderer(
-            xpad=16,
-            ypad=16,
+            xpad=8,
+            ypad=8,
             ellipsize=Pango.EllipsizeMode.END,
-            xalign=0.0,
-            width=220
+            xalign=0.0
         )
         type_renderer.connect('editing-started', self._on_editing_started, None)
         cols.pack_start(type_renderer, True)
         cols.add_attribute(type_renderer, "text", 0)
         cols.add_attribute(type_renderer, "editable", 1)
+        cols.set_cell_data_func(type_renderer, self._on_list_text_render)
         self.view.append_column(cols)
 
     @log
@@ -719,6 +617,14 @@ class PlaylistDialog():
         add_playlist_iter = self.model.append()
         self.model.set(add_playlist_iter, [0, 1], [_("New Playlist"), True])
 
+    @log
+    def _on_list_text_render(self, col, cell, model, _iter, data):
+        editable = model.get_value(_iter, 1)
+        if editable:
+            cell.add_class("dim-label")
+        else:
+            cell.remove_class("dim-label")
+
     @log
     def _on_selection(self, select_button):
         self.dialog_box.response(Gtk.ResponseType.ACCEPT)
@@ -748,3 +654,7 @@ class PlaylistDialog():
             playlist.create_playlist(editable.get_text())
             new_iter = self.model.insert_before(_iter)
             self.model.set(new_iter, [0, 1], [editable.get_text(), False])
+            self.view.set_cursor(self.model.get_path(new_iter),
+                                 self.view.get_columns()[0], False)
+            self.view.row_activated(self.model.get_path(new_iter),
+                                    self.view.get_columns()[0])
