@@ -45,12 +45,12 @@ import logging
 logger = logging.getLogger(__name__)
 
 playlist = Playlists.get_default()
-tracker = Tracker.SparqlConnection.get(None)
-
-if Gtk.get_minor_version() > 8:
-    from gi.repository.Gtk import Stack, StackTransitionType
-else:
-    from gi.repository.Gd import Stack, StackTransitionType
+try:
+    tracker = Tracker.SparqlConnection.get(None)
+except Exception as e:
+    from sys import exit
+    logger.error("Cannot connect to tracker, error '%s'\Exiting" % str(e))
+    exit(1)
 
 
 class Window(Gtk.ApplicationWindow):
@@ -71,6 +71,7 @@ class Window(Gtk.ApplicationWindow):
         selectNone.connect('activate', self._on_select_none)
         self.add_action(selectNone)
         self.set_size_request(200, 100)
+        self.set_icon_name('gnome-music')
 
         size_setting = self.settings.get_value('window-size')
         if isinstance(size_setting[0], int) and isinstance(size_setting[1], int):
@@ -178,16 +179,12 @@ class Window(Gtk.ApplicationWindow):
         self.selection_toolbar = SelectionToolbar()
         self.toolbar = Toolbar()
         self.views = []
-        self._stack = Stack(
-            transition_type=StackTransitionType.CROSSFADE,
+        self._stack = Gtk.Stack(
+            transition_type=Gtk.StackTransitionType.CROSSFADE,
             transition_duration=100,
             visible=True,
             can_focus=False)
-        if Gtk.get_minor_version() > 8:
-            self.set_titlebar(self.toolbar.header_bar)
-        else:
-            self._box.pack_start(self.toolbar.header_bar, False, False, 0)
-            self.set_hide_titlebar_when_maximized(True)
+        self.set_titlebar(self.toolbar.header_bar)
         self._box.pack_start(self.toolbar.searchbar, False, False, 0)
         self._box.pack_start(self._stack, True, True, 0)
         self._box.pack_start(self.player.eventBox, False, False, 0)
@@ -199,7 +196,7 @@ class Window(Gtk.ApplicationWindow):
             count = cursor.get_integer(0)
         if count > 0:
             self._switch_to_player_view()
-        #To revert to the No Music View when no songs are found
+        # To revert to the No Music View when no songs are found
         else:
             self._switch_to_empty_view()
 
@@ -317,7 +314,7 @@ class Window(Gtk.ApplicationWindow):
 
     @log
     def _on_notify_mode(self, stack, param):
-        #Slide out artist list on switching to artists view
+        # Slide out artist list on switching to artists view
         if stack.get_visible_child() == self.views[1] or \
            stack.get_visible_child() == self.views[3]:
             stack.get_visible_child().stack.set_visible_child_name('dummy')
